import os
import glob
import shutil
import argparse
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio import SeqFeature as SF
from Bio.SeqRecord import SeqRecord
from Bio.Alphabet import generic_dna
from Bio.SeqFeature import SeqFeature
from Bio.SeqFeature import FeatureLocation
import multiprocessing as mp
from datetime import datetime


Prodigal_parser_usage = '''
==================== Barrnap_Runner example commands ====================

# module needed
module load perl/5.28.0
module load hmmer/3.2.1
module load bedtools/2.27.1
module load barrnap/0.9

# for completed genome
BioSAK Barrnap_Runner -i genome_folder -x fna -t 6


barrnap MBAD.fna > barrnap_MBAD.gff3
bedtools getfasta -fi MBAD.fna -bed barrnap_MBAD.gff3 -fo rRNA_seq_MBAD.fasta -name -s

==========================================================================
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create)
    os.mkdir(folder_to_create)


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def export_aa_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.protein)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    SeqIO.write(seq_record, output_handle, 'fasta')


def Annotation_Barrnap(args):
    pass


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help="--", dest='subparser_name')

    # Annotation modules
    Prodigal_parser = subparsers.add_parser('Prodigal_Runner', description='Wrapper for running Prodigal',usage=Prodigal_parser_usage)

    Prodigal_parser.add_argument('-i',          required=True,  help='input genome folder')
    Prodigal_parser.add_argument('-x',          required=False, default='fna', help='file extension')
    Prodigal_parser.add_argument('-t',          required=False, type=int, default=1, help='number of threads')

    args = vars(parser.parse_args())

    Annotation_Barrnap(args)



import os


# Cdb_file =          '/Users/songweizhi/Desktop/Cdb.csv'
# bin_cluster_file =  '/Users/songweizhi/Desktop/bin_cluster.txt'



def Cdb_2_bin_cluster_file(Cdb_file, bin_cluster_file):

    cluster_to_bin_dict = {}
    obtained_clusters = set()
    for each_bin in open(Cdb_file):
        if not each_bin.startswith('genome,secondary_cluster'):
            each_bin_split = each_bin.strip().split(',')
            bin_id = each_bin_split[0]
            secondary_cluster = each_bin_split[1]

            obtained_clusters.add(secondary_cluster)

            if secondary_cluster not in cluster_to_bin_dict:
                cluster_to_bin_dict[secondary_cluster] = [bin_id]
            else:
                cluster_to_bin_dict[secondary_cluster].append(bin_id)

    obtained_clusters_list = sorted([i for i in obtained_clusters])

    bin_cluster_file_handle = open(bin_cluster_file, 'w')
    for j in obtained_clusters_list:
        bin_cluster_file_handle.write('cluster_%s\t%s\n' % (j, '\t'.join(cluster_to_bin_dict[j])))
    bin_cluster_file_handle.close()






#!/usr/bin/env python
from __future__ import division
import os
import argparse
from Bio import SeqIO
from datetime import datetime
from BioSAK.global_functions import time_format
from BioSAK.global_functions import sep_path_basename_ext


select_seq_usage = '''
=================== select_seq example commands ===================

# Extract sequences with provided id in seq_id.txt
BioSAK select_seq -seq ctg.fasta -id seq_id.txt -option 1

# Extract sequences except those in seq_id.txt
BioSAK select_seq -seq ctg.fasta -id seq_id.txt -option 0

# seq_id.txt file format (one id per line): 
NP_414542.1
NP_414548.1

===================================================================
'''

def select_seq(args):

    # read in argument
    seq_file = args['seq']
    id_file = args['id']
    select_option = args['option']

    # define output file name
    seq_file_path, seq_file_basename, seq_file_extension = sep_path_basename_ext(seq_file)
    output_file = '%s/%s_option_%s%s' % (seq_file_path, seq_file_basename, select_option, seq_file_extension)

    # report
    if select_option == 1:
        print(datetime.now().strftime(time_format) + 'Extracting sequences in %s' % id_file)
    if select_option == 0:
        print(datetime.now().strftime(time_format) + 'Extracting sequences except those in %s' % id_file)

    # get provided id list
    seq_id_list = []
    for seq_id in open(id_file):
        seq_id_list.append(seq_id.strip())

    # extract sequences
    output_file_handle = open(output_file, 'w')
    for seq in SeqIO.parse(seq_file, 'fasta'):
        seq_id = seq.id

        if select_option == 1:
            if seq_id in seq_id_list:
                SeqIO.write(seq, output_file_handle, 'fasta')

        if select_option == 0:
            if seq_id not in seq_id_list:
                SeqIO.write(seq, output_file_handle, 'fasta')

    output_file_handle.close()

    print(datetime.now().strftime(time_format) + 'Extracted sequences exported to %s' % output_file)
    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()

    # arguments for select_seq
    parser.add_argument('-seq',       required=True,            help='sequence file')
    parser.add_argument('-id',        required=True,            help='sequence id file, one id per line')
    parser.add_argument('-option',    required=True, type=int,  help='select option')

    args = vars(parser.parse_args())
    select_seq(args)



import os
import sys
import argparse
from itolapi import Itol, ItolExport


iTOL_usage = '''
=========================================== iTOL example commands ===========================================

BioSAK iTOL -tree Demo.tree -label Demo_label.txt -color Demo_color.txt -collapse Demo_collapse.txt

# Note:
1. iTOL module needs internet connection for working.
2. Some example input files can be found from the tutorial folder in BioSAK GitHub repository.
3. iTOL module doesn't show collapsed nodes. You should go to the provided link for better visualization. 

=============================================================================================================
'''

'''
# vertical_shift_factor
# horizontal_scale_factor
# leaf_sorting (disable Leaf sorting, important!!!)
'''

def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def iTOL(args):

    # read in arguments
    tree_file =         args['tree']
    collapse_file =     args['collapse']
    label_file =        args['label']
    color_file =        args['color']

    # define output file name
    tree_file_path, tree_file_basename, tree_file_extension = sep_path_basename_ext(tree_file)
    pwd_output_pdf = '%s/%s.pdf' % (tree_file_path, tree_file_basename)

    # Create the Itol class and add tree files
    my_job = Itol()
    my_job.add_file(tree_file)

    if color_file is not None:
        my_job.add_file(color_file)

    if label_file is not None:
        my_job.add_file(label_file)

    if collapse_file is not None:
        my_job.add_file(collapse_file)

    # upload tree
    print('Uploading tree. This may take some time depending on how large the tree is and how much load there is on the itol server')
    good_upload = my_job.upload()
    if not good_upload:
        print('There was an error:' + my_job.comm.upload_output)
        sys.exit(1)

    print('Tree Web Page URL: %s'     % my_job.get_webpage())

    # Export the tree above to pdf
    itol_exporter = my_job.get_itol_export()
    itol_exporter.set_export_param_value('format', 'pdf')
    itol_exporter.set_export_param_value('datasetList', 'dataset1')
    itol_exporter.export(pwd_output_pdf)

    print('Tree plot exported to %s.pdf' % tree_file_basename)


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()

    parser.add_argument('-tree',      required=True,  help='tree file in newick format')
    parser.add_argument('-label',     required=True,  help='label')
    parser.add_argument('-color',     required=False, help='color')
    parser.add_argument('-collapse',  required=False, help='collapse)')

    args = vars(parser.parse_args())

    iTOL(args)



#!/usr/bin/env python3

# Copyright (C) 2017, Weizhi Song, Kerrin Steensen and Torsten Thomas.
# songwz03@gmail.com and t.thomas@unsw.edu.au

# HgtSIM is free software: you can redistribute it and/or modify it under the terms of the GNU Affero
# General Public License as published by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.

# HgtSIM is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General
# Public License for more details.

# You should have received a copy of the GNU Affero General Public License along with this program.
# If not, see <http://www.gnu.org/licenses/>.


import os
import shutil
import random
import argparse
from Bio import SeqIO
from Bio.Seq import Seq
from Bio.SeqIO import FastaIO
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord
from Bio.Alphabet import generic_dna


def get_genome_size(fasta_file):
    genome = SeqIO.parse(fasta_file, 'fasta')
    total_length = 0
    for each_contig in genome:
        total_length += len(each_contig.seq)
    return total_length


def export_dna_record(gene_seq, gene_id, gene_description, output_handle):
    seq_object = Seq(gene_seq, IUPAC.unambiguous_dna)
    seq_record = SeqRecord(seq_object)
    seq_record.id = gene_id
    seq_record.description = gene_description
    fasta_out = FastaIO.FastaWriter(output_handle, wrap=None)
    fasta_out.write_header()
    fasta_out.write_record(seq_record)
    fasta_out.write_footer()


def simulate_reads(pwd_genome_file, read_number, read_length, insert_size, split):
    path, file_name = os.path.split(pwd_genome_file)
    genome_name, ext = os.path.splitext(file_name)

    if split == 1:
        output_r1 = '%s_R1.fasta' % (genome_name)
        output_r2 = '%s_R2.fasta' % (genome_name)
        output_r1_handle = open(output_r1, 'w')
        output_r2_handle = open(output_r2, 'w')
    else:
        output_combined = '%s_R12.fasta' % (genome_name)
        output_combined_handle = open(output_combined, 'w')

    seq = str(SeqIO.read(pwd_genome_file, 'fasta').seq)
    sequence_length = len(seq)
    fragment_length = 2 * read_length + insert_size

    n = 1
    while n <= read_number:
        rdm_num = random.randint(1, sequence_length)
        current_fragment = ''
        if (rdm_num + fragment_length) <= sequence_length:
            current_fragment = seq[rdm_num - 1: rdm_num + fragment_length - 1]
        elif (rdm_num + fragment_length) > sequence_length:
            seq_part_1_seq = seq[rdm_num - 1:]
            seq_part_2_seq = seq[:fragment_length - sequence_length + rdm_num - 1]
            current_fragment = seq_part_1_seq + seq_part_2_seq
        current_fragment_r1 = current_fragment[:read_length]
        current_fragment_r2 = current_fragment[-read_length:]
        current_fragment_r2_reverse_complement = str(Seq(current_fragment_r2, generic_dna).reverse_complement())
        current_read_r1_id = 'r%s_from_%s_%sth_bp_#0/1' % (n, genome_name, rdm_num)
        current_read_r2_id = 'r%s_from_%s_%sth_bp_#0/2' % (n, genome_name, rdm_num)

        current_read_r1_id = '%s_%sth_r1' % (genome_name, n)
        current_read_r2_id = '%s_%sth_r2' % (genome_name, n)

        if split == 1:
            export_dna_record(current_fragment_r1, current_read_r1_id, '', output_r1_handle)
            export_dna_record(current_fragment_r2_reverse_complement, current_read_r2_id, '', output_r2_handle)
        else:
            export_dna_record(current_fragment_r1, current_read_r1_id, '', output_combined_handle)
            export_dna_record(current_fragment_r2_reverse_complement, current_read_r2_id, '', output_combined_handle)
        n += 1
    if split == 1:
        output_r1_handle.close()
        output_r2_handle.close()
    else:
        output_combined_handle.close()


def Reads_simulator(args):

    rederence_genome = args['R']
    reads_number = args['n']
    reads_length = args['l']
    insert_size = args['i']
    split = args['split']

    simulate_reads(rederence_genome, reads_number, reads_length, insert_size, split)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument('-R', required=True, help='reference genomes')
    parser.add_argument('-n', required=True, type=int, help='reads number')
    parser.add_argument('-l', required=True, type=int, help='reads length')
    parser.add_argument('-i', required=True, type=int, help='insert size')
    parser.add_argument('-split', action="store_true", help='Export forward and reverse reads to separate files')

    args = vars(parser.parse_args())

    Reads_simulator(args)



import os
import shutil
import argparse
from datetime import datetime
import multiprocessing as mp


download_GenBank_genome_parser_usage = '''
================================== dwnld_GenBank_genome example commands ==================================

# Usage:
# 1. Go to https://www.ncbi.nlm.nih.gov/genome/browse#!/overview/
# 2. Search genomes you want to download (e.g. prokaryotes, proteobacteria or psychrobacter)
# 3. Click "Download" on the right side
# 4. provide the downloaded csv file with '-csv'

# Download all genomes in file prokaryotes.csv
BioSAK dwnld_GenBank_genome -csv prokaryotes.csv

# Only download genomes with provided accessions in accessions.txt
BioSAK dwnld_GenBank_genome -csv prokaryotes.csv -id accessions.txt

# accessions.txt file format 
# one accession per line, accessions can be found from the 6th column of the prokaryotes.csv file.
GCA_000265385.1
GCA_900176135
GCA_900107535.1


===========================================================================================================
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def sep_path_basename_ext(file_in):

    # separate path and file name
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'

    # separate file basename and extension
    file_basename, file_extension = os.path.splitext(file_name)

    return file_path, file_basename, file_extension


def genome_download_worker(argument_list):

    genome_record_split = argument_list[0]
    downloaded_genome_folder = argument_list[1]
    GenBank_FTP = genome_record_split[-2][1:-1]
    GenBank_FTP_id = GenBank_FTP.strip().split('/')[-1]
    assembly_id = genome_record_split[5]

    # download
    wget_cmd = 'wget %s/%s_genomic.fna.gz -P %s -q' % (GenBank_FTP, GenBank_FTP_id, downloaded_genome_folder)
    os.system(wget_cmd)

    # decompress
    os.system('gunzip %s/%s_genomic.fna.gz' % (downloaded_genome_folder, GenBank_FTP_id))

    # rename
    os.system('mv %s/%s_genomic.fna %s/%s.fna' % (downloaded_genome_folder, GenBank_FTP_id, downloaded_genome_folder, assembly_id))


def download_GenBank_genome(args):

    csv_file = args['csv']
    num_threads = args['t']
    time_format = '[%Y-%m-%d %H:%M:%S] '

    in_file_path, in_file_basename, in_file_extension = sep_path_basename_ext(csv_file)
    downloaded_genome_folder = '%s_genomes' % in_file_basename
    force_create_folder(downloaded_genome_folder)

    # download prokaryotes.txt file
    print(datetime.now().strftime(time_format) + 'Downloading ftp://ftp.ncbi.nlm.nih.gov//genomes/GENOME_REPORTS/prokaryotes.txt')
    wget_cmd = 'wget ftp://ftp.ncbi.nlm.nih.gov//genomes/GENOME_REPORTS/prokaryotes.txt -P %s' % downloaded_genome_folder
    #os.system(wget_cmd)

    # get id list to download
    genomes_to_download = []
    for each_genome in open(csv_file):

        if not each_genome.startswith('#Organism Name'):
            each_genome_name = each_genome.strip().split(',')[0]

            if each_genome_name.startswith('"'):
                each_genome_name = each_genome_name[1:-1]

                genomes_to_download.append(each_genome_name)

    print(genomes_to_download)

    # report
    print(datetime.now().strftime(time_format) + 'Downloading %s genomes with %s cores' % (len(genomes_to_download), num_threads))


    # download genome with multiprocessing
    pwd_prokaryotes_txt = '%s/prokaryotes.txt' % downloaded_genome_folder
    pwd_prokaryotes_txt = '/Users/songweizhi/Desktop/666666/prokaryotes.txt'

    list_for_multiple_arguments_download_worker = []
    for genome_record in open(pwd_prokaryotes_txt):

        genome_record_split = genome_record.strip().split('\t')

        genome_name = genome_record_split[0]



        if genome_name == 'synthetic Escherichia coli C321.deltaA':

            print(genome_record)

        if genome_name in genomes_to_download:



            assembly_id = genome_record_split[5]
            assembly_id_no_version = assembly_id.split('.')[0][1:]

            if assembly_id_no_version in genomes_to_download:
                list_for_multiple_arguments_download_worker.append([genome_record_split, downloaded_genome_folder])



    # run COG annotaion files with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(genome_download_worker, list_for_multiple_arguments_download_worker)
    pool.close()
    pool.join()


if __name__ == '__main__':

    parser = argparse.ArgumentParser()

    parser.add_argument('-csv', required=True,                          help='csv file from NCBI genome_browse')
    parser.add_argument('-t',   required=False, default=1, type=int,    help='number of threads')

    args = vars(parser.parse_args())

    download_GenBank_genome(args)



import argparse
from Bio import SeqIO
from BioSAK.global_functions import sep_path_basename_ext


rename_seq_usage = '''
========================= rename_ctg example commands =========================

# rename "NODE_941_length_17600_cov_52.7123" to "NODE_941"
BioSAK rename_seq -in Contigs.fa -sep_in "_" -n 2

# rename "Seawater|NODE|941|length|17600|cov|52.7123" to "Seawater_NODE_941"
BioSAK rename_seq -in Contigs.fa -sep_in "|" -sep_out "_" -n 3

# add prefix to all sequences in a fasta file
BioSAK rename_seq -in Contigs.fa -prefix seawater

# rename "NODE_941_length_17600_cov_52.7123" to "Seawater_NODE_941"
BioSAK rename_seq -in Contigs.fa -sep_in "_" -n 2 -prefix Seawater

===============================================================================
'''


def rename_seq(args):

    ctg_file_in =    args['in']
    sep_in =         args['sep_in']
    sep_out =        args['sep_out']
    column_to_keep = args['n']
    add_prefix =     args['prefix']

    ctg_file_path, ctg_file_basename, ctg_file_ext = sep_path_basename_ext(ctg_file_in)
    ctg_file_out = '%s/%s_renamed%s' % (ctg_file_path, ctg_file_basename, ctg_file_ext)

    if sep_out is None:
        sep_out = sep_in

    ctg_file_out_handle = open(ctg_file_out, 'w')
    for Seq_record in SeqIO.parse(ctg_file_in, 'fasta'):

        if (sep_in is not None) and (add_prefix is None):
            Seq_record_id_new = sep_out.join(Seq_record.id.split(sep_in)[:column_to_keep])

        elif (sep_in is None) and (add_prefix is not None):
            Seq_record_id_new = '%s_%s' % (add_prefix, Seq_record.id)

        elif (sep_in is not None) and (add_prefix is not None):
            Seq_record_id_new = '%s_%s' % (add_prefix, sep_out.join(Seq_record.id.split(sep_in)[:column_to_keep]))

        else:
            Seq_record_id_new = ''
            print('Don\'t know what to do, program exited!')
            exit()

        Seq_record.id = Seq_record_id_new
        SeqIO.write(Seq_record, ctg_file_out_handle, 'fasta')
    ctg_file_out_handle.close()


if __name__ == '__main__':

    rename_seq_parser = argparse.ArgumentParser()

    # arguments for rename_ctg_parser
    rename_seq_parser.add_argument('-in',         required=True,                          help='input sequence file')
    rename_seq_parser.add_argument('-sep_in',     required=False, default=None,           help='separator for input sequences')
    rename_seq_parser.add_argument('-sep_out',    required=False, default=None,           help='separator for output sequences, default: same as sep_in')
    rename_seq_parser.add_argument('-n',          required=False, default=None, type=int, help='the number of columns to keep')
    rename_seq_parser.add_argument('-prefix',     required=False, default=None,           help='add prefix to sequence')

    args = vars(rename_seq_parser.parse_args())

    rename_seq(args)




import os
import argparse
from Bio import SeqIO
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure


def take_kmer_mean(num_list, k_mer):

    k_mer_average_list = []
    n = 0

    while n < len(num_list):

        if n <= len(num_list) - k_mer:

            to_add_value = list(range(1, k_mer + 1))

            pos_list = []
            for each in to_add_value:
                pos_list.append(num_list[n + each - 1])

            # get average
            current_average = sum(pos_list) / k_mer
            k_mer_average_list.append(float("{0:.2f}".format(current_average)))

            n += 1

        elif (len(num_list) - k_mer) < n < len(num_list):
            k_mer_average_list.append(num_list[n])

            n += 1

    return k_mer_average_list


def plot_seq_depth(depth_file, seq_to_plot, start_pos, end_pos, bps_to_marker, plot_filename):
    #print('Extracting absolute depth from input file')
    x = []
    y = []
    bp_num = 1
    current_pos = 0
    for each_base in open(depth_file):
        each_base_split = each_base.strip().split('\t')
        seq_id = each_base_split[0]
        pos = int(each_base_split[1])
        depth = int(each_base_split[2])

        if seq_id == seq_to_plot:

            if pos < start_pos:
                pass

            elif pos == start_pos:
                x.append(pos)
                y.append(depth)
                current_pos = pos

            else:
                start_0 = None
                end_0 = None
                to_add = []

                if (pos == current_pos + 1) and (pos <= end_pos):
                    x.append(pos)
                    y.append(depth)
                    current_pos = pos

                elif (pos > current_pos + 1) and (pos <= end_pos):

                    # add zero
                    start_0 = current_pos + 1
                    end_0 = pos - 1
                    to_add = list(range(start_0, end_0 + 1))
                    for each_0 in to_add:
                        x.append(each_0)
                        y.append(0)

                    x.append(pos)
                    y.append(depth)
                    current_pos = pos

                elif (pos > current_pos + 1) and (pos > end_pos):

                    # add zero
                    start_0 = current_pos + 1
                    end_0 = end_pos
                    to_add = list(range(start_0, end_0 + 1))
                    for each_0 in to_add:
                        x.append(each_0)
                        y.append(0)

                    current_pos = pos

    #print('Calculating k-mer means')
    y = take_kmer_mean(y, k_mer)

    fig = plt.figure(figsize=(plot_width, plot_height), dpi=100)

    # Change the color and its transparency
    plt.plot(x, y, color="skyblue", alpha=0.7, linewidth=0.7)

    # titles
    plt.title(plot_filename,fontsize=7)
    plt.xlabel('Position (bp)', fontsize=10)
    plt.ylabel('Depth (X)', fontsize=10)

    plt.xticks(fontsize=7)
    plt.yticks(fontsize=7)

    ymax = 0
    if max(y) <= 2000:
        ymax = 2000
    elif (2000 < max(y) <= 5000):
        ymax = 5000
    elif (5000 < max(y) <= 10000):
        ymax = 10000
    else:
        ymax = max(y)

    plt.ylim(ymin=0, ymax=ymax)

    # add lines to specified positions
    if bps_to_marker != None:
        bps_to_marker_list = bps_to_marker.strip().split(',')
        for each_line in bps_to_marker_list:
            plt.axvline(x=int(each_line), c='red', linewidth=0.3)

    # Get plot
    plt.savefig('%s.png' % plot_filename, dpi=300)
    plt.close()


def plot_sam_depth(args):

    sequence_file = args['REF']
    depth_file = args['DEPTH']
    seq_to_plot = args['SeqID']
    start_pos = args['START']
    end_pos = args['END']
    k_mer = args['Kmer']
    plot_filename = args['Out']
    bps_to_marker = args['Lines']
    plot_width = args['xlen']
    plot_height = args['ylen']


    # get sequence length dict
    seq_id_length_dict = {}
    for each_seq in SeqIO.parse(sequence_file, 'fasta'):
        seq_id_length_dict[each_seq.id] = len(each_seq.seq)


    if (seq_to_plot != None) and (seq_to_plot not in seq_id_length_dict):
        print('Reference sequence %s not found in bam file, program exited!' % seq_to_plot)
        exit()


    # get depth_file base name
    depth_file_basename, depth_file_extension = os.path.splitext(depth_file)


    if seq_to_plot != None:

        # get the start and end of the region to plot
        if start_pos == None:
            start_pos = 1
        if end_pos == None:
            end_pos = seq_id_length_dict[seq_to_plot]

        print('Processing %s' % seq_to_plot)
        if plot_filename == None:
            plot_filename = '%s__%s__%s-%sbp__%smer' % (depth_file_basename, seq_to_plot, start_pos, end_pos, k_mer)
            plot_seq_depth(depth_file, seq_to_plot, start_pos, end_pos, bps_to_marker, plot_filename)

    if seq_to_plot == None:

        for each_ctg in SeqIO.parse(sequence_file, 'fasta'):
            print('Processing %s' % each_ctg.id)
            plot_filename = '%s__%s__%s-%sbp__%smer' % (depth_file_basename, each_ctg.id, 1, seq_id_length_dict[each_ctg.id], k_mer)
            plot_seq_depth(depth_file, each_ctg.id, 1, seq_id_length_dict[each_ctg.id], bps_to_marker, plot_filename)


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='', add_help=False)
    required = parser.add_argument_group('required arguments')
    optional = parser.add_argument_group('optional arguments')

    optional.add_argument('-h', action='help', help='Show this help message and exit')
    required.add_argument('-r', dest='REF',   nargs='?', required=True,  type=str, help='reference sequence file')
    required.add_argument('-d', dest='DEPTH', nargs='?', required=True,  type=str, help='depth file')
    optional.add_argument('-i', dest='SeqID', nargs='?', required=False, type=str, default=None, help='id of sequence to plot')
    optional.add_argument('-s', dest='START', nargs='?', required=False, type=int, default=None, help='start position to plot')
    optional.add_argument('-e', dest='END',   nargs='?', required=False, type=int, default=None, help='end position to plot')
    optional.add_argument('-k', dest='Kmer',  nargs='?', required=False, type=int, default=100, help='k-mer mean depth')
    optional.add_argument('-o', dest='Out',   nargs='?', required=False, type=str, default=None, help='output plot name')
    optional.add_argument('-l', dest='Lines', nargs='?', required=False, type=str, default=None, help='output plot name')
    optional.add_argument('-x', dest='xlen',  nargs='?', required=False, type=int, default=8, help='plot width')
    optional.add_argument('-y', dest='ylen',  nargs='?', required=False, type=int, default=3, help='plot height')

    args = vars(parser.parse_args())

    plot_sam_depth(args)



import os
import glob
import shutil
import argparse


split_folder_parser_usage = '''
===================================== split_folder example commands =====================================

BioSAK split_folder -in downloaded_genomes -x fasta -n 10

=========================================================================================================
'''


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create)
    os.mkdir(folder_to_create)


def split_folder(args):

    file_folder = args['in']
    file_ext = args['x']
    folder_num = int(args['n'])

    # create folder
    file_folder_sep = '%s_sep' % file_folder
    force_create_folder(file_folder_sep)


    # get file list
    file_name_re = '%s/*.%s' % (file_folder, file_ext)
    file_name_list = [os.path.basename(file_name) for file_name in glob.glob(file_name_re)]
    file_name_list = sorted(file_name_list)


    # get the number of file per folder
    file_num_per_folder = round(len(file_name_list)/folder_num)


    n = 1
    while n <= folder_num:

        # define current folder name
        current_folder_name = '%s_%s' % (file_folder, n)

        # get file list in current folder
        files_in_current_folder = []
        if n < folder_num:
            files_in_current_folder = file_name_list[(file_num_per_folder * (n - 1)):(file_num_per_folder * n)]
        else:
            files_in_current_folder = file_name_list[(file_num_per_folder * (n - 1)):]

        # create folder
        pwd_current_folder = '%s/%s' % (file_folder_sep, current_folder_name)
        mkdir_cmd = 'mkdir %s' % pwd_current_folder
        os.system(mkdir_cmd)

        # copy files to new folder
        for each_file in files_in_current_folder:
            pwd_file = '%s/%s' % (file_folder, each_file)
            pwd_file_sep = '%s/%s' % (file_folder_sep, each_file)
            cp_file_cmd = 'cp %s %s/' % (pwd_file, pwd_current_folder)
            os.system(cp_file_cmd)

        n += 1


if __name__ == "__main__":
    parser = argparse.ArgumentParser()

    parser.add_argument('-in', required=True, help='file folder')
    parser.add_argument('-x', required=True, help='file extension')
    parser.add_argument('-n', required=False, type=int, help='number of subfolder')

    args = vars(parser.parse_args())

    split_folder(args)
